import * as React from "react";
import * as Immutable from "immutable";
import ParMenu from "./ParMenu";
import { createImmutableMap } from "../createImmutaleMap";
const formatter = function (data, obj) {
    const { defaultMenuId, idField } = obj;
    return data.map(par => {
        par.selected = false;
        par.children.forEach(sub => {
            sub.selected = defaultMenuId === `${par[idField]}`;
        });
        return par;
    });
};
class NavMenu extends React.PureComponent {
    constructor(prop) {
        super(prop);
        this.changeImmutableData = (callback) => {
            this.setState(pre => {
                const { immutableData, selected } = pre;
                const { data, selected: _selected } = callback(immutableData, selected);
                return {
                    immutableData: data,
                    selected: _selected
                };
            });
        };
        this.state = this.initState(this.props);
    }
    static getDerivedStateFromProps(nextProps, preState) {
        if (nextProps.data !== preState.preData) {
            const data = formatter(nextProps.data, {
                defaultMenuId: nextProps.defaultMenuId,
                idField: nextProps.idField
            });
            return {
                preData: nextProps.data,
                immutableData: Immutable.fromJS(data),
                selected: nextProps.defaultMenuId || "",
            };
        }
        else {
            return null;
        }
    }
    initState(props) {
        const { defaultMenuId, data, idField, textField, urlField, iconField } = props;
        const newData = formatter(data, { idField: idField, defaultMenuId: defaultMenuId });
        return {
            selected: defaultMenuId || "",
            immutableData: Immutable.fromJS(newData),
            fieldObj: createImmutableMap({
                id: idField,
                text: textField,
                url: urlField,
                icon: iconField,
            }),
            preData: data,
        };
    }
    getParMenu() {
        const { immutableData, fieldObj } = this.state;
        const { idField, expand } = this.props;
        return immutableData.map((node, index) => {
            return (React.createElement(ParMenu, { key: node.get(idField), node: node, expand: expand, fieldObj: fieldObj, parIndex: index, changeData: this.changeImmutableData }));
        });
    }
    render() {
        const parItem = this.getParMenu();
        return (React.createElement("ul", { className: "g-menu" }, parItem));
    }
}
NavMenu.defaultProps = {
    textField: "name",
    idField: "id",
    iconField: "icon",
    urlField: "url",
};
export default NavMenu;
