import * as React from "react";
import * as Immutable from "immutable";
import * as ReactDom from "react-dom";
import { VelocityComponent } from "velocity-react";
import { SvgIcon } from "../my-icon/index";
class Notice extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = {
            messages: Immutable.List([]),
        };
        this.add = (obj) => {
            this.setState(pre => {
                return {
                    messages: pre.messages.push(obj),
                };
            });
        };
        this.clickFn = (e) => {
            const dom = e.currentTarget;
            const id = dom.dataset.id;
            this.remove(id);
        };
        this.remove = (id) => {
            this.setState(pre => {
                return {
                    messages: pre.messages.filter(val => {
                        return val.id !== id;
                    })
                };
            });
        };
        this.clear = () => {
            this.setState(pre => {
                return {
                    messages: pre.messages.clear(),
                };
            });
        };
    }
    render() {
        const { messages } = this.state;
        const list = messages.map(val => {
            const { id, text, type } = val;
            return (React.createElement(VelocityComponent, { key: id, animation: "transition.bounceRightIn", runOnMount: true },
                React.createElement("div", null,
                    React.createElement("span", { className: "g-alertInfo" },
                        React.createElement("span", null,
                            React.createElement("span", { className: `notice-item ${type}` },
                                React.createElement(SvgIcon, { className: type, size: "middle" })),
                            React.createElement("span", { className: "txt" }, text)),
                        React.createElement("span", { className: "m-alert-close", "data-id": id, onClick: this.clickFn },
                            React.createElement(SvgIcon, { className: "close" }))))));
        });
        return (React.createElement("div", { className: "g-notification" }, list));
    }
}
let noticeRef = null;
const createTimekey = function () {
    return new Date().getTime();
};
const fn = (ref) => {
    noticeRef = ref;
};
const createNotice = function (callback) {
    const wrap = document.getElementById("wrap-notice");
    ReactDom.render(React.createElement(Notice, { ref: fn }), wrap, function () {
        if (callback) {
            callback();
        }
    });
};
const notice = {
    add: function (text, type = "success", keep) {
        const id = `${createTimekey()}`;
        const obj = {
            id,
            type,
            text,
        };
        if (!noticeRef) {
            createNotice(function () {
                noticeRef.add(obj);
            });
        }
        else {
            noticeRef.add(obj);
        }
        if (!keep) {
            window.setTimeout(function () {
                noticeRef.remove(id);
            }, 2000);
        }
    },
    clear: function () {
        if (noticeRef) {
            noticeRef.clear();
        }
    },
};
export { createNotice };
export default notice;
